﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.Threading;
using System.IO;
using log4net;
using System.Reflection;
using System.Net;

namespace ADService
{
    /// <summary>
    /// 定时任务委托
    /// </summary>
    public delegate void TimerTaskDelegate();

    /// <summary>
    /// 定时任务服务类
    ///</summary> 
    public class TimerTask
    {
        private Int32 nCount = 0;

        private static ILog log4 = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        //字段  用来指定要执行的任务的方法
        private TimerTaskDelegate deleTimerTask;

        //构造函数  给deleTimerTask赋初值
        private TimerTask(TimerTaskDelegate _deleTimerTask)
        {
            deleTimerTask = _deleTimerTask;
        }

        //方法 定时任务，到达设定时间，执行deleTimerTask
        public void Start()
        {
            while (true)
            {

                int interval = Convert.ToInt32(ConfigurationManager.AppSettings["Interval"]);
                int hour = DateTime.Now.Hour;
                if (hour%interval == 0)
	            {
                    log4.Info("\r\n");
                    log4.Info("准备执行定时任务");
                    if (deleTimerTask != null)
                    {
                        deleTimerTask();
                        //访问目标页面
                        VisitPages();
                    }
                    Thread.Sleep(30000);
	            }

                Thread.Sleep(1000);
                nCount++;
                if (nCount==30)
                {
                     log4.Info("In TimeTask!");
                }
            }
        }

        //静态方法 返回执行定时任务的线程
        public static Thread CreateTimerTaskThread(TimerTaskDelegate _deleTimerTask)
        {
            //创建TimerTaskService类的实例
            TimerTask timerTaskService = new TimerTask(_deleTimerTask);
            //创建启动线程
            Thread ThreadTimerTaskService = new Thread(new ThreadStart(timerTaskService.Start));
            return ThreadTimerTaskService;
        }


        //Modify by Lu jinlong 2013.02.28 
        /// <summary>
        /// 访问配置好的多个Web页面
        /// </summary>
        private void VisitPages()
        {
            string page = ConfigurationManager.AppSettings["PageUrls"];
            if (!string.IsNullOrEmpty(page))
            {
                string[] pageArr = page.Split(';');
                for (int i = 0; i < pageArr.Length; i++)
                {
                    string pageUrl = pageArr[i].Trim();
                    if (!string.IsNullOrEmpty(pageUrl))
                    {
                        try
                        {
                            string msg = HttpConnectToServer(pageUrl);
                            log4.Info(pageUrl + ":" + msg + "\r\n");
                        }
                        catch (Exception)
                        {
                            log4.Error("访问页面出错，请检查配置的页面" + pageUrl + "的地址是否正确" + "\r\n");
                        }
                    }
                }
            }
        }
        //发送消息到服务器
        public string HttpConnectToServer(string ServerPage)
        {
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(ServerPage);
            try
            {
                WebResponse response = request.GetResponse();
                return "ok";
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}